"""
Dynamically generated options dialogs for Fabber models and inference methods

Copyright (c) 2016-2018 University of Oxford, Martin Craig
"""
import logging

from PySide import QtGui, QtCore

from quantiphyse.gui.widgets import OverlayCombo

from .option_widgets import get_option_widget, get_label

#: Maximum number of instances of 'numbered' options (e.g. PSP_byname<n>) """
NUMBERED_OPTIONS_MAX = 20

LOG = logging.getLogger(__name__)

def _del(rundata, key):
    if key in rundata: del rundata[key]

class OptionsDialog(QtGui.QDialog):
    """
    QtGui.QDialog which displays dynamically generated Fabber options (e.g. for method or model)
    """
    
    #: Signal when option value is changed
    sig_changed = QtCore.Signal(str)

    def __init__(self, parent=None, title="Options", desc="", **kwargs):
        super(OptionsDialog, self).__init__(parent)
        vbox = QtGui.QVBoxLayout()
        self.setLayout(vbox)

        self.title_label = get_label(title, size=20)
        vbox.addWidget(self.title_label)
        
        self.desc_label = get_label(desc, bold=True, italic=True)
        vbox.addWidget(self.desc_label)

        self.scroll_area = QtGui.QScrollArea()
        self.scroll_area.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setMinimumHeight(500)
        self.scroll_area_content = QtGui.QWidget()
        self.grid = QtGui.QGridLayout(self.scroll_area_content)
        self.scroll_area.setWidget(self.scroll_area_content)
        vbox.addWidget(self.scroll_area)

        button_box = QtGui.QDialogButtonBox()
        button_box.setOrientation(QtCore.Qt.Horizontal)
        button_box.setStandardButtons(QtGui.QDialogButtonBox.Close)
        button_box.accepted.connect(self.accept)
        button_box.rejected.connect(self.reject)
        vbox.addWidget(button_box)

        self.desc_first = kwargs.get("desc_first", False)
        self.ivm = kwargs.get("ivm", None)
        self.rundata = kwargs.get("rundata", None)
        self.options = []
        self.ignore_opts = set()
        self.option_widgets = []

    def fit_width(self):
        """
        Adjust the width of the dialog to fit the required options plus a possible scrollbar
        """
        width = self.scroll_area_content.minimumSizeHint().width() + self.scroll_area.verticalScrollBar().width()
        self.scroll_area.setMinimumWidth(max(width, 600))
        
    def set_title(self, title, desc=""):
        """ 
        Set title and optional description for the dialog 
        """
        self.title_label.setText(title)
        self.desc_label.setText(desc)

    def set_options(self, opts):
        """
        Set the options to be displayed in the dialog

        :param opts: Options as sequence of options dictionary, as returned by Fabber.get_options()
        """
        self._clear_options()
        self.options = [o for o in opts if o["name"] not in self.ignore_opts]
        self._create_opts_ui()

    def ignore(self, *opts):
        """
        Ignore certain options

        :param opts: Sequence of names of options to ignore
        """
        for opt in opts:
            self.ignore_opts.add(opt)

    def _create_opts_ui(self):
        req = [opt for opt in self.options if not opt["optional"]]
        nonreq = [opt for opt in self.options if opt["optional"]]
        
        if req:
            label = get_label("Mandatory options", size=14, bold=True)
            self.grid.addWidget(label, 0, 0)
            self._add_opts(req, 1)
        if nonreq:
            label = get_label("Non-mandatory options", size=12, bold=True)
            self.grid.addWidget(label, len(req)+1, 0)
            self._add_opts(nonreq, len(req)+2)

        self.grid.setAlignment(QtCore.Qt.AlignTop)
        self.adjustSize()
        
    def _clear_options(self):
        self._del_layout(self.grid)

    def _del_layout(self, layout):
        while True:
            item = layout.takeAt(0)
            if item is None:
                break
            elif item.widget() is None:
                self._del_layout(item)    
            else:
                item.widget().deleteLater()
                
    def _add_opts(self, opts, startrow):
        row = 0
        for opt in opts:
            if opt["name"].find("<n>") >= 0 and opt["type"] not in ("INT", "FLOAT"):
                # This is a numbered option. Create multiple widgets, each dependent on the previous
                opt_base = opt["name"][:opt["name"].find("<n>")]
                opt_suffix = opt["name"][opt["name"].find("<n>") + 3:]
                prev = None
                for n in range(1, NUMBERED_OPTIONS_MAX+1):
                    newopt = dict(opt)
                    newopt["name"] = "%s%i%s" % (opt_base, n, opt_suffix)
                    widget = self._add_option_widget(newopt, row+startrow)
                    if prev is not None:
                        prev.add_dependent(widget)
                    prev = widget
                    row += 1
            else:
                self._add_option_widget(opt, row+startrow)
                row += 1
            
        return row

    def _add_option_widget(self, opt, row):
        widget = get_option_widget(opt, options=self.rundata, ivm=self.ivm, desc_first=self.desc_first)
        widget.update(self.rundata)
        widget.add(self.grid, row)
        # Need to keep in scope or they get GC'd! FIXME this suggests the design is rubbish
        self.option_widgets.append(widget)
        return widget

class PriorsDialog(OptionsDialog):
    """
    OptionsDialog which displays parameter priors
    """

    TITLE = "Model parameter priors"
    DESC = "Describes optional prior information about each model parameter"
        
    def __init__(self, parent=None, **kwargs):
        super(PriorsDialog, self).__init__(parent, title=self.TITLE, desc=self.DESC, **kwargs)
        self.priors = []
        self.prior_widgets = []
        self.overlays = []
        self.params = []
        
    def set_params(self, params):
        """
        Set the model parameters

        :param params: Sequence of parameter names
        """
        LOG.debug("Params=%s", params)
        self.params = params
        self._repopulate()

    def _get_prior_idx(self, param):
        prior_idx = 1
        while "PSP_byname%i" % prior_idx in self.rundata:
            if param == self.rundata["PSP_byname%i" % prior_idx]:
                return prior_idx
            prior_idx += 1
        return -1

    def _get_widgets(self, idx):
        prior_idx = self._get_prior_idx(self.params[idx])
        ptype = self.rundata.get("PSP_byname%i_type" % prior_idx, "N")
        image = self.rundata.get("PSP_byname%i_image" % prior_idx, "")
        mean = self.rundata.get("PSP_byname%i_mean" % prior_idx, "")
        prec = self.rundata.get("PSP_byname%i_prec" % prior_idx, "")
        trans = self.rundata.get("PSP_byname%i_transform" % prior_idx, "")

        type_combo = QtGui.QComboBox()
        type_combo.addItem("Normal", "N")
        type_combo.addItem("Image Prior", "I")
        type_combo.addItem("Spatial prior, type M", "M")
        type_combo.addItem("Spatial prior , type P", "P")
        type_combo.addItem("Spatial prior, type m", "m")
        type_combo.addItem("Spatial prior , type p", "p")
        type_combo.addItem("ARD prior", "A")
        type_combo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        type_combo.setCurrentIndex(type_combo.findData(ptype))
        type_combo.currentIndexChanged.connect(self._changed)
        
        image_combo = OverlayCombo(self.ivm, static_only=True)
        image_combo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        if image != "":
            image_combo.setCurrentIndex(image_combo.findText(image))
        image_combo.currentIndexChanged.connect(self._changed)
        
        mean_cb = QtGui.QCheckBox()
        mean_cb.setChecked(ptype != "I" and mean != "")
        mean_cb.stateChanged.connect(self._changed)

        mean_edit = QtGui.QLineEdit(mean)
        mean_edit.editingFinished.connect(self._changed)
        
        prec_cb = QtGui.QCheckBox()
        prec_cb.setChecked(prec != "")
        prec_cb.stateChanged.connect(self._changed)

        prec_edit = QtGui.QLineEdit(prec)
        prec_edit.editingFinished.connect(self._changed)
        
        trans_combo = QtGui.QComboBox()
        trans_combo.addItem("Default transformation", "")
        trans_combo.addItem("No transformation", "I")
        trans_combo.addItem("Log transformation", "L")
        trans_combo.addItem("Soft-plus transformation", "S")
        trans_combo.addItem("Fractional transformation", "F")
        trans_combo.setSizeAdjustPolicy(QtGui.QComboBox.AdjustToContents)
        trans_combo.setCurrentIndex(trans_combo.findData(trans))
        trans_combo.currentIndexChanged.connect(self._changed)

        return (type_combo, 
                QtGui.QLabel("Image: "), 
                image_combo, 
                mean_cb, 
                QtGui.QLabel("Custom mean: "), 
                mean_edit,
                prec_cb, 
                QtGui.QLabel("Custom precision: "), 
                prec_edit,
                trans_combo)

    def _update_widgets(self):
        for idx in range(len(self.params)):
            w = self.prior_widgets[idx]
        
            prior_type = w[0].itemData(w[0].currentIndex())

            # Image overlay selector
            need_image = (prior_type == "I")
            w[1].setEnabled(need_image)
            w[2].setEnabled(need_image)

            # Custom mean
            custom_mean_allowed = prior_type != "I"
            w[3].setEnabled(custom_mean_allowed)
            w[4].setEnabled(custom_mean_allowed and w[3].isChecked())
            w[5].setEnabled(custom_mean_allowed and w[3].isChecked())

            # Custom precision
            custom_prec_allowed = True
            w[6].setEnabled(custom_prec_allowed)
            w[7].setEnabled(custom_prec_allowed and w[6].isChecked())
            w[8].setEnabled(custom_prec_allowed and w[6].isChecked())

    def _changed(self):
        self._update_widgets()
        self._update_rundata()

    def _update_rundata(self):
        prior_idx = 1
        for idx, param in enumerate(self.params):
            w = self.prior_widgets[idx]
            prior_type = w[0].itemData(w[0].currentIndex())
            
            # Do we need to set any options for this prior?
            if prior_type != "N" or w[3].isChecked() or w[6].isChecked() or w[9].currentIndex() > 0:
                self.rundata["PSP_byname%i" % prior_idx] = param
                self.rundata["PSP_byname%i_type" % prior_idx] = prior_type
                if prior_type == "I":
                    self.rundata["PSP_byname%i_image" % prior_idx] = w[2].currentText()
                else:
                    _del(self.rundata, "PSP_byname%i_image" % prior_idx)
                if w[3].isChecked() and prior_type != 'I':
                    self.rundata["PSP_byname%i_mean" % prior_idx] = w[5].text()
                else:
                    _del(self.rundata, "PSP_byname%i_mean" % prior_idx)
                if w[6].isChecked():
                    self.rundata["PSP_byname%i_prec" % prior_idx] = w[8].text()
                else:
                    _del(self.rundata, "PSP_byname%i_prec" % prior_idx)
                if w[9].currentIndex() > 0:
                    self.rundata["PSP_byname%i_transform" % prior_idx] = w[9].itemData(w[9].currentIndex())
                prior_idx += 1

        # Get rid of any subsequent PSP options that were previously set
        while "PSP_byname%i" % prior_idx in self.rundata:
            _del(self.rundata, "PSP_byname%i" % prior_idx)
            _del(self.rundata, "PSP_byname%i_type" % prior_idx)
            _del(self.rundata, "PSP_byname%i_image" % prior_idx)
            _del(self.rundata, "PSP_byname%i_prec" % prior_idx)
            _del(self.rundata, "PSP_byname%i_transform" % prior_idx)
            prior_idx += 1
        
    def _repopulate(self):
        self._clear_options()
        self.grid.setSpacing(20)
        self.prior_widgets = []
        
        if not self.params:
            self.grid.addWidget(QtGui.QLabel("No parameters found! Make sure model is properly configured"))

        for idx, param in enumerate(self.params):
            self.prior_widgets.append(self._get_widgets(idx))
        
            self.grid.addWidget(QtGui.QLabel("%s: " % param), idx, 0)
            for col, w in enumerate(self.prior_widgets[idx]):
                self.grid.addWidget(w, idx, col+1)

        self._update_widgets()
        self.grid.setAlignment(QtCore.Qt.AlignTop)
        self.adjustSize()
        


"""
Widgets for controlling different types of Fabber options

Copyright (c) 2016-2018 University of Oxford, Martin Craig
"""

import os
import logging

from PySide import QtCore, QtGui

from quantiphyse.gui.widgets import OverlayCombo
from quantiphyse.gui.options import NumberListOption

LOG = logging.getLogger(__name__)

def get_label(text="", size=None, bold=False, italic=False):
    """
    :return: QtGui.QLabel with specified text and attributes
    """
    label = QtGui.QLabel(text)
    font = label.font()
    font.setBold(bold)
    font.setItalic(italic)
    label.setFont(font)
    if size: label.setStyleSheet("QLabel{ font: %ipx }" % size)
    return label

class OptionWidget(QtCore.QObject):
    """
    Widget which allows a Fabber option to be controlled
    """
    def __init__(self, opt, **kwargs):
        super(OptionWidget, self).__init__()
        self.key = opt["name"]
        self.dtype = opt["type"]
        self.req = not opt["optional"]
        self.default = opt["default"]
        self.desc = opt["description"]

        self.options = kwargs.get("options", {})
        self.ivm = kwargs.get("ivm", None)
        self.desc_first = kwargs.get("desc_first", False)
        self.dependents = []
        self.widgets = []

        # Basic label
        self.label = get_label(opt["name"])
        self.widgets.append(self.label)

        # Description label
        self.desclabel = get_label(opt["description"])
        self.desclabel.setToolTip("--%s" % self.key)
        self.desclabel.resize(400, self.desclabel.height())
        self.desclabel.setWordWrap(True)
        self.widgets.append(self.desclabel)

        # For non-mandatory options, provide a checkbox to enable them
        if self.req:
            self.enable_cb = None
            self.checked = True
        else:
            self.enable_cb = QtGui.QCheckBox()
            self.enable_cb.stateChanged.connect(self._checkbox_toggled)
            self.widgets.append(self.enable_cb)
            self.checked = False

    def update(self, options):
        """
        Update the displayed option value from an options dictionary
        """
        val = options.get(self.key, None)
        if val is not None:
            self.set_value(val)
            if self.enable_cb is not None:
                self.enable_cb.setChecked(True)
        else:
            self.set_value(self.default)
            if self.enable_cb is not None:
                self.enable_cb.setChecked(False)
        # Make sure visibility is updated even if state is unchanged
        if self.enable_cb is not None: 
            self._checkbox_toggled()

    def update_options(self):
        """
        Update the options dictionary from the current displayed value
        """
        if self.checked:
            self.options[self.key] = self.get_value()
        elif self.key in self.options:
            del self.options[self.key]
        LOG.debug(self.options)

    def set_value(self, value):
        """ Set the displayed option value """
        pass
    
    def get_value(self):
        """ Get the displayed option value """
        return ""

    def set_visible(self, visible=True):
        """ Set the visibility of the option widget """
        for widget in self.widgets:
            widget.setVisible(visible)

    def set_enabled(self, enabled=True):
        """ Set the enabled (greyed out) status of the option widget """
        for widget in self.widgets:
            widget.setEnabled(enabled)
        # Checkbox is always enabled!
        if self.enable_cb is not None: 
            self.enable_cb.setEnabled(True)

    def add(self, grid, row):
        """ 
        Add the option widget to a grid layout
        
        This base method adds the label/description and checkbox if required.
        Subclasses should call this method first and then add their own specific
        widget to column 1 of the grid

        :param grid: QtGui.QGridLayout 
        :param row: Row index number
        """
        if self.desc_first:
            label = self.desclabel
        else:
            label = self.label
            grid.addWidget(self.desclabel, row, 2)

        if self.req:
            grid.addWidget(label, row, 0)
        else:
            hbox = QtGui.QHBoxLayout()
            hbox.addWidget(self.enable_cb)
            hbox.addWidget(label, row)
            grid.addLayout(hbox, row, 0)

    def add_dependent(self, dep):
        """
        Add another OptionWidget as a dependent

        Dependent widgets are only visible when the parent widget is enabled/checked
        """
        if not self.enable_cb: return
        self.dependents.append(dep)
        dep.set_visible(self.checked)
        if not self.checked: dep.enable_cb.setChecked(False)

    def _checkbox_toggled(self):
        # This function is only called if we have a checkbox
        self.checked = self.enable_cb.checkState() == QtCore.Qt.CheckState.Checked
        self.set_enabled(self.checked)
        
        for dep in self.dependents:
            dep.set_visible(self.checked)
            if not self.checked: dep.enable_cb.setChecked(False)

        self.update_options()
        
class IntegerOptionWidget(OptionWidget):
    """
    Option which allows an integer to be chosen using a spin box
    """
    def __init__(self, opt, **kwargs):
        OptionWidget.__init__(self, opt, **kwargs)
        self.spin = QtGui.QSpinBox()
        self.spin.valueChanged.connect(self.update_options)
        self.widgets.append(self.spin)
    
    def get_value(self):
        return str(self.spin.value())
        
    def set_value(self, value):
        if value == "":
            try:
                self.spin.setValue(int(self.default))
            except ValueError:
                self.spin.setValue(0)
        else:
            self.spin.setValue(int(value))

    def add(self, grid, row):
        OptionWidget.add(self, grid, row)
        grid.addWidget(self.spin, row, 1)

class StringOptionWidget(OptionWidget):
    """
    OptionWidget allowing a text string to be entered
    """
    def __init__(self, opt, **kwargs):
        OptionWidget.__init__(self, opt, **kwargs)
        self.edit = QtGui.QLineEdit()
        self.edit.editingFinished.connect(self.update_options)
        self.widgets.append(self.edit)
            
    def get_value(self):
        return self.edit.text()
        
    def set_value(self, value):
        self.edit.setText(value)

    def add(self, grid, row):
        OptionWidget.add(self, grid, row)
        grid.addWidget(self.edit, row, 1)

class FileOptionWidget(StringOptionWidget):
    """
    OptionWidget allowing a file name to be chosen using a standard file selector
    """
    def __init__(self, opt, **kwargs):
        StringOptionWidget.__init__(self, opt, **kwargs)
        self.hbox = QtGui.QHBoxLayout()
        self.hbox.addWidget(self.edit)
        self.btn = QtGui.QPushButton("Choose")
        self.hbox.addWidget(self.btn)
        self.widgets.append(self.btn)
        self.btn.clicked.connect(self._choose_file)
    
    def _choose_file(self):
        dialog = QtGui.QFileDialog()
        dialog.setFileMode(QtGui.QFileDialog.AnyFile)
        if dialog.exec_():
            fname = dialog.selectedFiles()[0]
            self.edit.setText(fname)
            self.update_options()

    def add(self, grid, row):
        OptionWidget.add(self, grid, row)
        grid.addLayout(self.hbox, row, 1)

class VestParseError(Exception):
    """ Failure to parse a VEST matrix file """
    pass

class MatrixFileOptionWidget(FileOptionWidget):
    """
    Option which allows the user to choose a file containing a matrix
    """
    def __init__(self, opt, **kwargs):
        FileOptionWidget.__init__(self, opt, **kwargs)
        edit_btn = QtGui.QPushButton("Edit")
        self.hbox.addWidget(edit_btn)
        self.widgets.append(edit_btn)
        edit_btn.clicked.connect(self._edit_file)
    
    def _read_vest(self, fname):
        f = None
        in_matrix = False
        mat = []
        try:
            f = open(fname, "r")
            lines = f.readlines()
            nx, ny = 0, 0
            for line in lines:
                if in_matrix:
                    nums = [float(num) for num in line.split()]
                    if len(nums) != nx: raise VestParseError("Incorrect number of x values")
                    mat.append(nums)
                elif line.startswith("/Matrix"):
                    if nx == 0 or ny == 0: raise VestParseError("Missing /NumWaves or /NumPoints")
                    in_matrix = True
                elif line.startswith("/NumWaves"):
                    parts = line.split()
                    if len(parts) == 1: raise VestParseError("No number following /NumWaves")
                    nx = int(parts[1])
                elif line.startswith("/NumPoints") or line.startswith("/NumContrasts"):
                    parts = line.split()
                    if len(parts) == 1: raise VestParseError("No number following /NumPoints")
                    ny = int(parts[1])
            if len(mat) != ny:
                raise VestParseError("Incorrect number of y values")      
        finally:
            if f is not None: f.close()

        if not in_matrix: 
            raise VestParseError("File '%s' does not seem to contain a VEST matrix" % fname)
        else:
            return mat, ""

    def _read_ascii(self, fname):
        f = None
        in_matrix = False
        mat = []
        desc = ""
        try:
            f = open(fname, "r")
            lines = f.readlines()
            nx = 0
            for line in lines:
                if line.strip().startswith("#"):
                    desc += line.lstrip("#")
                else:
                    row = [float(n) for n in line.split()]
                    if not in_matrix:
                        nx = len(row)
                        in_matrix = True
                    elif len(row) != nx:
                        raise Exception("Incorrect number of x values: %s" % line)
                    mat.append(row)
        finally:
            if f is not None: f.close()

        return mat, desc

    def _write_vest(self, fname, matrix):
        f = None
        try:
            f = open(fname, "w")
            nx, ny = len(matrix), len(matrix[0])
            f.write("/NumWaves %i\n" % nx)
            f.write("/NumPoints %i\n" % ny)
            f.write("/Matrix\n")
            for row in matrix:
                for item in row:
                    f.write("%f " % item)
                f.write("\n")
        finally:
            if f is not None: f.close()

    def _write_ascii(self, fname, matrix, desc=""):
        f = None
        try:
            f = open(fname, "w")
            for line in desc.splitlines():
                f.write("#%s\n" % line)
            for row in matrix:
                f.write(" ".join([str(v) for v in row]))
                f.write("\n")
        finally:
            if f is not None: f.close()

    def _edit_file(self):
        fname = self.edit.text()
        if fname.strip() == "":
            msg_box = QtGui.QMessageBox()
            msg_box.setText("Enter a filename")
            msg_box.setStandardButtons(QtGui.QMessageBox.Ok)
            msg_box.exec_()
            return
        elif not os.path.exists(fname):
            msg_box = QtGui.QMessageBox()
            msg_box.setText("File does not exist - create?")
            msg_box.setStandardButtons(QtGui.QMessageBox.Ok | QtGui.QMessageBox.Cancel)
            msg_box.setDefaultButton(QtGui.QMessageBox.Ok)
            ret = msg_box.exec_()
            if ret != QtGui.QMessageBox.Ok:
                return
            open(fname, "a").close()

        try:
            mat, desc = self._read_vest(fname)
            ascii = False
        except VestParseError:
            mat, desc = self._read_ascii(fname)
            ascii = True
        self.mat_dialog.set_matrix(mat, desc)
        if self.mat_dialog.exec_():
            mat, desc = self.mat_dialog.get_matrix()
            if ascii:
                self._write_ascii(fname, mat, desc)
            else:
                self._write_vest(fname, mat)

class ImageOptionWidget(OptionWidget):
    """
    OptionWidget subclass which allows image options to be chosen
    from the current list of overlays
    """
    def __init__(self, opt, **kwargs):
        OptionWidget.__init__(self, opt, **kwargs)
        self.combo = OverlayCombo(self.ivm, static_only=True)
        self.widgets.append(self.combo)

    def get_value(self):
        return self.combo.currentText()

    def set_value(self, value):
        idx = self.combo.findText(value)
        self.combo.setCurrentIndex(idx)

    def add(self, grid, row):
        OptionWidget.add(self, grid, row)
        grid.addWidget(self.combo, row, 1)

class ListOptionWidget(OptionWidget):
    """
    OptionWidget subclass for a list of values
    """
    def __init__(self, opt, **kwargs):
        OptionWidget.__init__(self, opt, **kwargs)
        self.edit = NumberListOption(intonly=opt["type"] == "INT")
        self.edit.sig_changed.connect(self.update_options)
        self.widgets.append(self.edit)

    def get_value(self):
        return self.edit.value
        
    def set_value(self, value):
        self.edit.value = value

    def add(self, grid, row):
        OptionWidget.add(self, grid, row)
        grid.addWidget(self.edit, row, 1)

    def update_options(self):
        """
        Update the options dictionary from the current displayed value
        """
        if self.checked:
            values = self.get_value()
            for idx, value in enumerate(values):
                key = self.key.replace("<n>", str(idx+1))
                self.options[key] = value
        else:
            idx = 1
            while 1:
                key = self.key.replace("<n>", str(idx))
                if key not in self.options:
                    break
                del self.options[key]
        LOG.debug(self.options)

OPT_VIEW = {
    "INT" : IntegerOptionWidget,
    "BOOL": OptionWidget,
    "FILE": FileOptionWidget,
    "IMAGE": ImageOptionWidget,
    "TIMESERIES" : ImageOptionWidget,
    "MVN": FileOptionWidget,
    "MATRIX": MatrixFileOptionWidget,
}

def get_option_widget(opt, **kwargs):
    """
    Get an option widget for a Fabber option

    :param opt: Option as a dictionary, as returned by Fabber.get_options
    """

    if opt["name"].find("<n>") >= 0:
        return ListOptionWidget(opt, **kwargs)
    else:
        return OPT_VIEW.get(opt["type"], StringOptionWidget)(opt, **kwargs)
   